@@ -95,16 +95,31 @@ snapshot_formatters = [
             'description about region box (what and how it represents)'),
         Formatter('<temperature-sz histogram>',
                   lambda snapshot, record, fmt:
-                  temperature_sz_hist_str(snapshot, record, fmt),
+                  temperature_sz_hist_str(snapshot, record, fmt, False),
                   'temperature to total size of the regions histogram'),
+        Formatter('<temperature-df-passed-sz histogram>',
+                  lambda snapshot, record, fmt:
+                  temperature_sz_hist_str(snapshot, record, fmt, True),
+                  ' '.join(['temperature to total size of DAMOS filters (df)',
+                            'passed regions histogram'])),
         Formatter('<recency-sz histogram>',
                   lambda snapshot, record, fmt:
-                  recency_hist_str(snapshot, record, fmt),
+                  recency_hist_str(snapshot, record, fmt, False),
                   'last accessed time to total size of the regions histogram'),
+        Formatter('<recency-df-passed-sz histogram>',
+                  lambda snapshot, record, fmt:
+                  recency_hist_str(snapshot, record, fmt, True),
+                  ' '.join(['last accessed time to total size of',
+                            'DAMOS filters (df) passed regions histogram'])),
+
         Formatter('<heatmap>',
                   lambda snapshot, record, fmt:
                   heatmap_str(snapshot, record, fmt),
                   'heatmap of the snapshot'),
+        Formatter(
+            '<filters passed type>',
+            lambda snapshot, record, fmt: filters_pass_type_of(record),
+            'type of <filters passed bytes> memory'),
         ]
 
 region_formatters = [
@@ -150,10 +165,6 @@ region_formatters = [
             _damo_fmt_str.format_sz(region.sz_filter_passed, fmt.raw_number)
             if region.sz_filter_passed is not None else 'N/A',
             'bytes of the region that passed DAMOS filters'),
-        Formatter(
-            '<filters passsed type>',
-            lambda index, region, fmt: fmt.region_filters_pass_type,
-            'type of <filters passed bytes> memory'),
         Formatter(
             '<box>',
             lambda index, region, fmt:
@@ -166,7 +177,13 @@ default_snapshot_head_format = 'monitored time: [<start time>, <end time>] (<dur
 default_snapshot_head_format_without_heatmap = 'monitored time: [<start time>, <end time>] (<duration>)'
 default_region_format = '<index> addr <start address> size <size> access <access rate> age <age>'
 
-def temperature_sz_hist_str(snapshot, record, fmt):
+def filters_pass_type_of(record):
+    ops_filters = [f for f in record.scheme_filters if f.handled_by_ops()]
+    if len(ops_filters) == 0:
+        return 'n/a'
+    return ', '.join(['%s' % f for f in ops_filters])
+
+def temperature_sz_hist_str(snapshot, record, fmt, df_passed_sz):
     raw = fmt.raw_number
     if len(snapshot.regions) == 0:
         return 'no region in snapshot'
@@ -177,7 +194,10 @@ def temperature_sz_hist_str(snapshot, record, fmt):
         temperature = temperature_of(region, fmt.temperature_weights)
         if not temperature in hist:
             hist[temperature] = 0
-        hist[temperature] += region.size()
+        if df_passed_sz:
+            hist[temperature] += region.sz_filter_passed
+        else:
+            hist[temperature] += region.size()
 
     temperatures = sorted(hist.keys())
     min_temp, max_temp = temperatures[0], temperatures[-1]
@@ -219,7 +239,7 @@ def temperature_sz_hist_str(snapshot, record, fmt):
         lines.append('%s %s %s' % (trange_str, sz_str, bar))
     return '\n'.join(lines)
 
-def recency_hist_str(snapshot, record, fmt):
+def recency_hist_str(snapshot, record, fmt, df_passed_sz):
     raw = fmt.raw_number
     if len(snapshot.regions) == 0:
         return 'no region in snapshot'
@@ -230,7 +250,10 @@ def recency_hist_str(snapshot, record, fmt):
             last_used = 0
         if not last_used in hist:
             hist[last_used] = 0
-        hist[last_used] += region.size()
+        if df_passed_sz is True:
+            hist[last_used] += region.sz_filter_passed
+        else:
+            hist[last_used] += region.size()
 
     last_used_times = sorted(hist.keys())
     min_lut, max_lut = last_used_times[0], last_used_times[-1]
@@ -775,7 +798,7 @@ def pr_records(fmt, records):
     else:
         _damo_print.pr_with_pager_if_needed(fmt_records(fmt, records))
 
-class RecordsVisualizationFormat:
+class ReportFormat:
     sort_regions_by = None
     sort_regions_dsc = None
     temperature_weights = None
@@ -805,7 +828,6 @@ class RecordsVisualizationFormat:
     json = None
     raw = None
 
-    region_filters_pass_type = None
 
     @classmethod
     def from_args(cls, args):
@@ -831,11 +853,6 @@ class RecordsVisualizationFormat:
         self.raw_number = args.raw_number
         self.json = args.json
         self.raw = args.raw_form
-        type_str, err = _damon_args.passed_bytes_type_str(args.damos_filter)
-        if err is not None:
-            self.region_filters_pass_type = err
-        else:
-            self.region_filters_pass_type = type_str
         return self
 
     def total_sz_only(self):
@@ -901,67 +918,90 @@ class RecordsVisualizationFormat:
         self.json = kvpairs['json']
         return self
 
-    def runtime_update(self, records):
-        if self.format_record_head == None:
-            if len(records) > 1:
-                self.format_record_head = default_record_head_format
-            else:
-                self.format_record_head = ''
+def has_ops_filters(records):
+    for record in records:
+        for scheme_filter in record.scheme_filters:
+            if scheme_filter.handled_by_ops():
+                return True
+    return False
 
-        if self.format_snapshot_head == None:
-            need_snapshot_head = False
-            for record in records:
-                if len(record.snapshots) > 1:
-                    need_snapshot_head = True
-                    break
-            if need_snapshot_head:
-                self.format_snapshot_head = default_snapshot_head_format
-            else:
-                self.format_snapshot_head = 'heatmap: <heatmap>'
+def set_formats_hist_style(args, fmt, records):
+    if args.style == 'temperature-sz-hist':
+        legend = '<temperature>'
+        hist_keyword = '<temperature-sz histogram>'
+        filter_passed_hist_keyword = '<temperature-df-passed-sz histogram>'
+    else:
+        # args.style == 'recency-sz-hist':
+        legend = '<last accessed time (us)>'
+        hist_keyword = '<recency-sz histogram>'
+        filter_passed_hist_keyword = '<recency-df-passed-sz histogram>'
 
+    snapshot_head_content = []
+    if has_ops_filters(records):
+        snapshot_head_content += [
+                '# damos filters (df): <filters passed type>',
+                '%s <df-passed size>' % legend, filter_passed_hist_keyword, '']
+    snapshot_head_content += ['%s <total size>' % legend, hist_keyword]
+    fmt.format_snapshot_head = '\n'.join(snapshot_head_content)
+    fmt.format_region = ''
 
-def set_formats(args):
+def set_formats(args, records):
+    fmt = ReportFormat.from_args(args)
     if args.style == 'simple-boxes':
-        args.format_snapshot_head = default_snapshot_head_format_without_heatmap
-        args.format_region = '<box> size <size> access rate <access rate> age <age>'
-        args.region_box_min_max_height = [1, 1]
-        args.region_box_min_max_length = [1, 40]
-        args.region_box_align = 'right'
-        args.region_box_colorset = 'emotion'
-    elif args.style == 'temperature-sz-hist':
-        args.format_snapshot_head = '\n'.join([
-            '<temperature> <total size>',
-            '<temperature-sz histogram>'])
-        args.format_region = ''
-    elif args.style == 'recency-sz-hist':
-        args.format_snapshot_head = '\n'.join([
-            '<last accessed time (us)> <total size>',
-            '<recency-sz histogram>'])
-        args.format_region = ''
+        fmt.format_snapshot_head = default_snapshot_head_format_without_heatmap
+        fmt.format_region = '<box> size <size> access rate <access rate> age <age>'
+        fmt.region_box_min_max_height = [1, 1]
+        fmt.region_box_min_max_length = [1, 40]
+        fmt.region_box_align = 'right'
+        fmt.region_box_colorset = 'emotion'
+    elif args.style in ['temperature-sz-hist', 'recency-sz-hist']:
+        set_formats_hist_style(args, fmt, records)
 
-    args.region_box_values = [v if v != 'none' else None
+    fmt.region_box_values = [v if v != 'none' else None
             for v in args.region_box_values]
 
     if args.total_sz_only:
-        args.format_snapshot_head = ''
-        args.format_region = ''
-        args.format_snapshot_tail = '<total bytes>'
+        fmt.format_snapshot_head = ''
+        fmt.format_region = ''
+        fmt.format_snapshot_tail = '<total bytes>'
 
     if args.region_box:
-        if args.region_box_min_max_height[1] > 1:
-            args.format_region = '<box>%s' % default_region_format
+        if fmt.region_box_min_max_height[1] > 1:
+            fmt.format_region = '<box>%s' % default_region_format
         else:
-            args.format_region = '<box>\n%s' % default_region_format
-        if args.format_snapshot_tail.find('<region box description>') == -1:
-            args.format_snapshot_tail = ('%s\n<region box description>' %
-                    args.format_record_tail)
+            fmt.format_region = '<box>\n%s' % default_region_format
+        if fmt.format_snapshot_tail.find('<region box description>') == -1:
+            fmt.format_snapshot_tail = ('%s\n<region box description>' %
+                    fmt.format_record_tail)
+    if len(records) == 0:
+        return fmt
 
-    if args.format_region == default_region_format:
-        if args.tried_regions_of or args.damos_filter:
-            args.format_region += ' %s %s' % (
-                    '<filters passsed type>', '<filters passed bytes>')
+    if fmt.format_record_head == None:
+        if len(records) > 1:
+            fmt.format_record_head = default_record_head_format
+        else:
+            fmt.format_record_head = ''
 
-    return RecordsVisualizationFormat.from_args(args)
+    if fmt.format_region == default_region_format:
+        for record in records:
+            if len(record.scheme_filters) > 0:
+                fmt.format_region += ' df-passed <filters passed bytes>'
+                break
+
+    if fmt.format_snapshot_head == None:
+        need_snapshot_head = False
+        for record in records:
+            if len(record.snapshots) > 1:
+                need_snapshot_head = True
+                break
+        if need_snapshot_head:
+            fmt.format_snapshot_head = default_snapshot_head_format
+        else:
+            fmt.format_snapshot_head = 'heatmap: <heatmap>'
+    if '<filters passed bytes>' in fmt.format_region:
+        fmt.format_snapshot_head += '\n# damos filters (df): <filters passed type>'
+
+    return fmt
 
 def handle_ls_keywords(args):
     if args.ls_record_format_keywords:
@@ -1010,10 +1050,9 @@ def main(args):
         if os.path.isfile(fmt_string):
             with open(fmt_string, 'r') as f:
                 fmt_string = f.read()
-        fmt = RecordsVisualizationFormat.from_kvpairs(json.loads(fmt_string))
+        fmt = ReportFormat.from_kvpairs(json.loads(fmt_string))
     else:
-        fmt = set_formats(args)
-    fmt.runtime_update(records)
+        fmt = set_formats(args, records)
     for record in records:
         try:
             pr_records(fmt, records)
